(function($, undefined) {
    if ($.apta !== undefined) $.error("jquery-apta has already been loaded!");
	
	var $document = $(document),
        apta;
	
    $.apta = apta = {

        linkClickSelector: "a[data-confirm], a[data-method], a[data-remote], a[data-disable-with]",
		
        buttonClickSelector: "button[data-remote]",
		
        inputChangeSelector: "select[data-remote], input[data-remote], textarea[data-remote]",
		
        formSubmitSelector: "form",
		
        formInputClickSelector: "form input[type=submit], form input[type=image], form button[type=submit], form button:not([type])",
		
        disableSelector: "input[data-disable-with], button[data-disable-with], textarea[data-disable-with]",
		
        enableSelector: "input[data-disable-with]:disabled, button[data-disable-with]:disabled, textarea[data-disable-with]:disabled",
		
        requiredInputSelector: "input[name][required]:not([disabled]),textarea[name][required]:not([disabled])",
		
        fileInputSelector: "input[type=file]",
		
        linkDisableSelector: "a[data-disable-with]",

        CSRFProtection: function(xhr) {
            var token = $('meta[name="csrf-token"]').attr("content");
            if (token) xhr.setRequestHeader("X-CSRF-Token", token);
        },

        fire: function(obj, name, data) {
            var event = $.Event(name);
            obj.trigger(event, data);
            return event.result !== false;
        },

        href: function(element) {

            return element.attr("href");
        },

        confirm: function(message) {

            return confirm(message);
        },

        ajax: function(opts) {

            return $.ajax(opts);
        },

        handleRemote: function(element) {
            var method, url, data, elCrossDomain, crossDomain, withCredentials, dataType, options;

            if (this.fire(element, "ajax:before")) {
                elCrossDomain = element.data("cross-domain");
                crossDomain = elCrossDomain === undefined ? null : elCrossDomain;
                withCredentials = element.data("with-credentials") || null;
                dataType = element.data("type") || ($.ajaxSettings && $.ajaxSettings.dataType);

                if (element.is("form")) {
                    method = element.attr("method");
                    url = element.attr("action");
                    data = element.serializeArray();
                    var button = element.data("apta:submit-button");
                    if (button) {
                        data.push(button);
                        element.data("apta:submit-button", null);
                    }
                } else if (element.is(this.inputChangeSelector)) {
                    method = element.data("method");
                    url = element.data("url");
                    data = element.serialize();
                    if (element.data("params")) data = data + "&" + element.data("params");
                } else if (element.is(this.buttonClickSelector)) {
                    method = element.data("method") || "get";
                    url = element.data("url");
                    data = element.serialize();
                    if (element.data("params")) data = data + "&" + element.data("params");
                } else {
                    method = element.data("method");
                    url = this.href(element);
                    data = element.data("params") || null;
                }

                options = {
                    type: method || "GET",
                    data: data,
                    dataType: dataType,
					
                    beforeSend: function(xhr, settings) {
                        if (settings.dataType === undefined) {
                            xhr.setRequestHeader("accept", "*/*;q=0.5, " + settings.accepts.script);
                        }
                        return this.fire(element, "ajax:beforeSend", [xhr, settings]);
                    },
                    success: function(data, status, xhr) {
                        element.trigger("ajax:success", [data, status, xhr]);
                    },
                    complete: function(xhr, status) {
                        element.trigger("ajax:complete", [xhr, status]);
                    },
                    error: function(xhr, status, error) {
                        element.trigger("ajax:error", [xhr, status, error]);
                    },
                    crossDomain: crossDomain
                };

                if (withCredentials) {
                    options.xhrFields = {
                        withCredentials: withCredentials
                    };
                }

                if (url) options.url = url;

                var jqxhr = this.ajax(options);
                element.trigger('ajax:send', jqxhr);
				
                return jqxhr;
            } else {
                return false;
            }
        },

        handleMethod: function(link) {
            var href = this.href(link),
                method = link.data("method"),
                target = link.attr("target"),
                csrfToken = $("meta[name=csrf-token]").attr("content"),
                csrfParam = $("meta[name=csrf-param]").attr("content"),
                form = $('<form method="post" action="' + href + '"></form>'),
                metadataInput = '<input name="method" value="' + method + '" type="hidden" />';

            if (csrfParam !== undefined && csrfToken !== undefined) {
                metadataInput += '<input name="' + csrfParam + '" value="' + csrfToken + '" type="hidden" />';
            }

            if (target) form.attr("target", target);

            form.hide().append(metadataInput).appendTo("body");
            form.submit();
        },

        disableFormElements: function(form) {
            form.find(this.disableSelector).each(function() {
                var element = $(this),
                    method = element.is("button") ? "html" : "val";

                element.data("apta:enable-with", element[method]());
                element[method](element.data("disable-with"));
                element.prop("disabled", true);
            });
        },

        enableFormElements: function(form) {
            form.find(this.enableSelector).each(function() {
                var element = $(this),
                    method = element.is("button") ? "html" : "val";

                if (element.data("apta:enable-with")) element[method](element.data("apta:enable-with"));

                element.prop("disabled", false);
            });
        },

        allowAction: function(element) {
            var message = element.data("confirm"),
                answer = false,
                callback;

            if (!message) {
                return true;
            }
            if (this.fire(element, "confirm")) {
                answer = this.confirm(message);
                callback = this.fire(element, "confirm:complete", [answer]);
            }
            return answer && callback;
        },

        blankInputs: function(form, specifiedSelector, nonBlank) {
            var inputs = $(),
                input, valueToCheck,
                selector = specifiedSelector || "input,textarea",
                allInputs = form.find(selector);

            allInputs.each(function() {
                input = $(this);
                valueToCheck = input.is("input[type=checkbox],input[type=radio]") ? input.is(":checked") : input.val();

                if (!valueToCheck === !nonBlank) {

                    if (input.is("input[type=radio]") && allInputs.filter('input[type=radio]:checked[name="' + input.attr("name") + '"]').length) {
                        return true;
                    }

                    inputs = inputs.add(input);
                }

                return true;
            });

            return inputs.length ? inputs : false;
        },

        nonBlankInputs: function(form, specifiedSelector) {

            return this.blankInputs(form, specifiedSelector, true);
        },

        stopEverything: function(e) {
            $(e.target).trigger("apta:everythingStopped");
            e.stopImmediatePropagation();

            return false;
        },

        disableElement: function(element) {
            element.data("apta:enable-with", element.html());
            element.html(element.data("disable-with"));

            element.bind("click.aptaDisable", function(e) {
                return this.stopEverything(e);
            });
        },

        enableElement: function(element) {
            if (element.data("apta:enable-with") !== undefined) {
                element.html(element.data("apta:enable-with"));
                element.removeData("apta:enable-with");
            }
            element.unbind("click.aptaDisable");
        }
    };


    if (apta.fire($document, "apta:attachBindings")) {

        $.ajaxPrefilter(function(options, originalOptions, xhr) {
            if (!options.crossDomain) {
                apta.CSRFProtection(xhr);
            }
        });

        $document.delegate(apta.linkDisableSelector, "ajax:complete", function() {
            apta.enableElement($(this));
        });

        $document.delegate(apta.linkClickSelector, "click.apta", function(e) {
            var link = $(this),
                method = link.data("method"),
                data = link.data("params"),
                metaClick = e.metaKey || e.ctrlKey;

            if (!apta.allowAction(link)) return apta.stopEverything(e);
            if (!metaClick && link.is(apta.linkDisableSelector)) apta.disableElement(link);

            if (link.data("remote") !== undefined) {
                if (metaClick && (!method || method === "GET") && !data) return true;
                var handleRemote = apta.handleRemote(link);

                if (handleRemote === false) {
                    apta.enableElement(link);
                } else {
                    handleRemote.error(function() {
                        apta.enableElement(link);
                    });
                }

                return false;

            } else if (link.data("method")) {
                apta.handleMethod(link);
                return false;
            }

            return true;
        });

        $document.delegate(apta.buttonClickSelector, "click.apta", function(e) {
            var button = $(this);
            if (!apta.allowAction(button)) return apta.stopEverything(e);

            apta.handleRemote(button);
            return false;
        });

        $document.delegate(apta.inputChangeSelector, "change.apta", function(e) {
            var link = $(this);
            if (!apta.allowAction(link)) return apta.stopEverything(e);

            apta.handleRemote(link);
            return false;
        });

        $document.delegate(apta.formSubmitSelector, "submit.apta", function(e) {
            var form = $(this),
                remote = form.data("remote") !== undefined,
                blankRequiredInputs = apta.blankInputs(form, apta.requiredInputSelector),
                nonBlankFileInputs = apta.nonBlankInputs(form, apta.fileInputSelector);

            if (!apta.allowAction(form)) return apta.stopEverything(e);
            if (blankRequiredInputs && form.attr("novalidate") == undefined && apta.fire(form, "ajax:aborted:required", [blankRequiredInputs])) return apta.stopEverything(e);

            if (remote) {
                if (nonBlankFileInputs) {
                    setTimeout(function() {
                        apta.disableFormElements(form);
                    }, 13);
                    var aborted = apta.fire(form, "ajax:aborted:file", [nonBlankFileInputs]);

                    if (!aborted) {
                        setTimeout(function() {
                            apta.enableFormElements(form);
                        }, 13);
                    }

                    return aborted;
                }

                apta.handleRemote(form);

                return false;

            } else {
                setTimeout(function() {
                    apta.disableFormElements(form);
                }, 13);
            }

            return true;
        });

        $document.delegate(apta.formInputClickSelector, "click.apta", function(event) {
            var button = $(this);

            if (!apta.allowAction(button)) {
                apta.stopEverything(event);
                return;
            }

            var name = button.attr("name"),
                data = name ? {name: name, value: button.val()} : null;

            button.closest("form").data("apta:submit-button", data);
        });

        $document.delegate(apta.formSubmitSelector, "ajax:beforeSend.apta", function(event) {
            if (this == event.target) apta.disableFormElements($(this));
        });

        $document.delegate(apta.formSubmitSelector, "ajax:complete.apta", function(event) {
            if (this == event.target) apta.enableFormElements($(this));
        });

        $(function() {
            var csrfToken = $("meta[name=csrf-token]").attr("content"),
                csrfParam = $("meta[name=csrf-param]").attr("content");
			
            $('form input[name="' + csrfParam + '"]').val(csrfToken);
        });
    }
}(jQuery));